// Copyright (c) .NET Foundation. All rights reserved.
// Licensed under the Apache License, Version 2.0. See License.txt in the project root for license information.

using JetBrains.Annotations;
using System;

namespace Microsoft.Data.Entity.Infrastructure
{
    /// <summary>
    ///     <para>
    ///         Interface for extensions that are stored in <see cref="DbContextOptions.Extensions"/>. 
    ///     </para>
    ///     <para>
    ///         This interface is typically used by database providers (and other extensions). It is generally
    ///         not used in application code.
    ///     </para>
    /// </summary>
    public interface IDbContextOptionsExtension
    {
        /// <summary>
        ///     Adds the services required to make the selected options work. This is used when there is no external <see cref="IServiceProvider"/> 
        ///     and EF is maintaining its own service provider internally. This allows database providers (and other extensions) to register their
        ///     required services when EF is creating an service provider.
        /// </summary>
        /// <param name="builder"> The builder to add services to. </param>
        void ApplyServices([NotNull] EntityFrameworkServicesBuilder builder);
    }
}
