// Copyright (c) .NET Foundation. All rights reserved.
// Licensed under the Apache License, Version 2.0. See License.txt in the project root for license information.

namespace Microsoft.Data.Entity.Metadata
{
    /// <summary>
    ///     Indicates when a value for a property will be generated by the database. Even when a
    ///     property is set to be generated by the database, EF may still attempt to save a specific value (rather than
    ///     having one generated by the database) when an entity is added and a value is assigned, or the property is  
    ///     marked as modified for an existing entity.
    /// </summary>
    public enum ValueGenerated
    {
        /// <summary>
        ///     A value is never generated by the database. 
        /// </summary>
        Never,

        /// <summary>
        ///     A value is generated by the database when an entity is added. The most common scenario for this
        ///     is generated primary key values.
        /// </summary>
        OnAdd,

        /// <summary>
        ///     A value is generated every time an entity is saved. This is typically used for scenarios such as row 
        ///     versions, timestamps, etc.
        /// </summary>
        OnAddOrUpdate
    }
}
